#!/usr/bin/env perl

use 5.010001;
# use utf8;
use utf8::all;
use strict;
use warnings;
use warnings FATAL => 'utf8';
use autodie;

# use open qw[ :utf8 :std ];

use BibTeX::Parser;
use Getopt::Long::Descriptive qw[describe_options prog_name];
use Path::Tiny qw[path];

sub compress {
  my($str) = @_;
  $str =~ s{\s+}{ }g;
  $str =~ s{\A\s+}{};
  $str =~ s{\s+\z}{};
  return $str;
}

my $default_sep = $ENV{BIB_SUBTITLE_SEP} // q{. };

my($opt, $usage) = describe_options(
  '[perl] %c %o [>utf8-output.bib]',
  [ 'input-file|i=s', 'Source BibTeX file name.', {required => 1} ],
  [ 'output-file|o=s', 'Target BibTeX file name.',  ],
  [ 'input-encoding|e=s', 'Input file encoding.', +{ default => 'UTF-8' }, ],
  [ 'output-encoding|E=s', 'Output file encoding.', +{ default => 'UTF-8' }, ],
  ['-e and -E values may be Perl binmode string containing ":".'],
  [ 'join-title|j', 'Join title and subtitle into title.' ],
  [ 'sep|s=s',
    compress(
      q{Separator between title and subtitle.
      Defaults to env var BIB_SUBTITLE_SEP or ". ".
      Must explicitly contain any whitespace!}
    ),
    +{ default => $default_sep },
  ],
  [ 'decode|d', 'Replace LaTeX macros with Unicode characters.',
    +{default => 1},
  ],
  [ 'no-decode|D', 'Do not replace LaTeX macros with Unicode characters.', +{implies => +{decode=>0}}, ],
  [ 'decode-german|g', 'Decode with german conventions. ("a for \"a etc.)', +{implies=>+{decode=>1}}],
  [ field_capitalization => 'hidden', +{default => 'Lowercase'} ],
  [ _field_capitalization => [
      [ 'fields-lower-case|l', 'Make field names lowercase. (Default.)', +{ implies => +{ field_capitalization => 'Lowercase'} }, ],
      [ 'fields-title-case|t', 'Make field names titlecase.', +{ implies => +{ field_capitalization => 'Titlecase'} }, ],
      [ 'fields-upper-case|u', 'Make field names uppercase.', +{ implies => +{ field_capitalization => 'Uppercase'} }, ],
    ],
  ],
  [ type_capitalization => 'hidden', +{default => 'Titlecase'} ],
  [ _type_capitalization => [
      [ 'types-lower-case|L', 'Make type names lowercase.', +{ implies => +{ type_capitalization => 'Lowercase'} }, ],
      [ 'types-title-case|T', 'Make type names titlecase. (Default.)', +{ implies => +{ type_capitalization => 'Titlecase'} }, ],
      [ 'types-upper-case|U', 'Make type names uppercase.', +{ implies => +{ type_capitalization => 'Uppercase'} }, ],
    ],
  ],
  [ 'canonize-names|n', 'Print author and editor fields in canonical BibTeX format.'],
  [ 'help|h', 'Print usage help and exit.', +{shortcircuit => 1} ],
  +{
    show_defaults => 1,
    getopt_conf => [qw(bundling no_auto_abbrev no_ignore_case)],
  },
);

if ( $opt->help ) {
  my $prog = prog_name;
  print "$prog - Clean up BibTeX sources with BibTeX::Parser\n\n";
  print $usage->text;
  exit;
}

my $source = $opt->input_file;
my $dest   = $opt->output_file;

my $sep = $opt->sep;

my $source_enc = $opt->input_encoding;
my $dest_enc = $opt->output_encoding;
for my $enc ( $source_enc, $dest_enc ) {
  $enc = ":encoding($enc)" unless $enc =~ /:/;
}

for my $path ( $source, $dest ) {
  $path // next;
  $path = path $path;
}

die "Input file and output file must be different\n"
  if $dest and $source->absolute eq $dest->absolute;

my %to_string_opts = map {; $_ => $opt->$_ } qw(
  field_capitalization
  type_capitalization
  canonize_names
);

my %decode_opts = ( german => $opt->decode_german );

my %decode_method = (
  author => 'cleaned_author',
  editor => 'cleaned_editor',
);

my $dest_fh   = $dest ? $dest->openw($dest_enc) : \*STDOUT;
my $source_fh = $source->openr($source_enc);

my $parser = BibTeX::Parser->new($source_fh);

my $count = 0;
while ( my $entry = $parser->next ) {
  $count++;
  die "Error parsing entry #$count " . $entry->error unless $entry->parse_ok;
  if ( $opt->join_title ) {
    if ( $entry->has('title') and $entry->has('subtitle') ) {
      my $title    = $entry->field('title');
      my $subtitle = $entry->field('subtitle');
      delete $entry->{subtitle};
      $entry->field( title => $title . $sep . $subtitle );
    }
  }
  if ( $opt->decode ) {
    while ( my ( $field, $method ) = each %decode_method ) {
      $entry->$field( $entry->$method(%decode_opts) );
    }
    for my $field ( $entry->fieldlist ) {
      $entry->field( $field, $entry->cleaned_field( $field, %decode_opts ) );
    }
  }
  say {$dest_fh} $entry->to_string(%to_string_opts);
}


# This software is copyright (c) 2022 by Benct Philip Jonsson.
# 
# This is free software; you can redistribute it and/or modify it under
# the same terms as the Perl 5 programming language system itself.
# 
# http://dev.perl.org/licenses/
